
#### GEOTYPING ###################################################################################
  
# This section:
#    1. Matches ABS SA2 data about division area with population data 
#    2. Calculates population density for each division
#    3. Assigns each division a geotype based on its population density

##################################################################################################

# Import area data
SA2 <- read.csv("SA2_2011_AUST.csv")
  
# Get State Codes for later reference (Output: STATE.CODES) 
STATE.CODES <- select (SA2, STATE_CODE_2011, STATE_NAME_2011)
STATE.CODES <- distinct(STATE.CODES)
colnames(STATE.CODES) <-c('STATE','STATE.NAME')

# Area/size of division (Output: SA2$AREA)
SA2 <- select(SA2, SA2_MAINCODE_2011, STATE_CODE_2011, AREA_ALBERS_SQM)   
colnames(SA2) <-c('ID','STATE','AREA')
SA2$AREA <- SA2$AREA / 1000000              # AREA IS NOW IN SQ KMS
  
# Import population data population in each division (Output: SA2.POP)
SA2.POP <- read.xlsx(xlsxFile = "SA2POPULATION.xlsx", sheet = 1, startRow = 7, skipEmptyRows = FALSE)
# N.B. Some versions of open.xlsx read in an empty second column, so this assignment of colnames is a bit
# round-about to account for that
colnames(SA2.POP)[1] <- "ID"
colnames(SA2.POP)[ncol(SA2.POP)] <- "POPULATION"
#select relevant columns
SA2.POP <- select(SA2.POP, ID, POPULATION)
# remove annotations at the bottom of the data
SA2.POP <- subset(SA2.POP, POPULATION >= 0) 
SA2.POP[,1] <- sapply(SA2.POP[,1] , as.numeric)


# Merge area and population data ------------------------------------------

# Merge area and population data (Output: SA2)
SA2 <- left_join(SA2, SA2.POP, by = "ID")
rm(SA2.POP)

# Calculate population density
SA2$DENSITY <- SA2$POPULATION / SA2$AREA
SA2$DENSITY <- sapply(as.vector(SA2["DENSITY"]), as.numeric)
  
# Fill missing values: some divisions have missing values for area or population
SA2$AREA[is.na(SA2$AREA)] <- 0
SA2$POPULATION[is.na(SA2$POPULATION)] <- 0
SA2$DENSITY[is.na(SA2$DENSITY)] <- 0

# Assign geotypes ---------------------------------------------------------

# Read geotype parameters
GEOTYPES <- read.xlsx(xlsxFile = INPUTS, sheet = "Geotypes", startRow = 1, skipEmptyRows = FALSE)
# Select and name relevant columns
GEOTYPE.CODES <- select(GEOTYPES, GEOTYPE, NAME)
colnames(GEOTYPE.CODES) <-c('GEOTYPE','GEOTYPE.NAME')
  
# Get geotype thresholds
geotype.thresholds <- as.vector(GEOTYPES[,'VALUE'])

# Assign geotypes to SA2s (Output: sA2$Geotypes)  
# This function sorts SA2s into 'bins', depending on population density
SA2$GEOTYPE <- cut(SA2$DENSITY, breaks = c(geotype.thresholds, Inf), labels = 1:nrow(GEOTYPES))
SA2$GEOTYPE <- sapply(as.vector(SA2["GEOTYPE"]), as.numeric)
  
rm(geotype.thresholds, GEOTYPES)

# Calculate total area for each state-geotype class -----------------------

# get summary statistics for the area and population of each state-geotype class
RAN.AREA <- SA2 %>%
    group_by(STATE, GEOTYPE) %>% 
    summarise_each(funs(Sum=sum(.)), AREA, POPULATION) %>% 
    ungroup(.) 

# write RAN.AREA to csv 
write.csv(RAN.AREA, "SUMMARY.GEOTYPE.csv", row.names=FALSE)

rm(SA2)
  


