#### RADIO ACCESS NETWORK (RAN) DIMENSIONING #################################

   # This section determines the number of sites in the RAN over time. The 
   # number of sites is determined in one of two ways:
   #   1. It is calculated using a ground-up network dimensioning approach (via the RAN Dimensioning function).
   #   2. It is specified exogenously (for example, by reference to the number of MNO sites)


# Read area of each state-geotype class -----------------------------------

RAN.AREA <- read.csv("SUMMARY.GEOTYPE.csv")


# Define RAN Dimensioning function ----------------------------------------

# This function calculates the number of sites required for each state-geotype 
# class for each year (t) in the time horizon.

RAN.DIMENSIONING <- function(t){

  ### COVERAGE DIMENSIONING ###
  
  # This section calculates the number of coverage cells for each state-geotype class
  
  # Define a function for the area of a circle
  circ<- function(x){(x^2)*pi}
    
  # import cell parameters and select relevant columns
  Cell.Parameters <- read.xlsx(xlsxFile = INPUTS, sheet = "Cell.Parameters", startRow = 1, skipEmptyRows = FALSE)
  Cell.Parameters <- select(Cell.Parameters, GEOTYPE, CEDR, INDOOR, CELL.OVERLAP)
    
  # import lookup tables
  Cell.Radius <- read.xlsx(xlsxFile = "CELLRADIUS.xlsx", sheet = "DATA", startRow = 1, skipEmptyRows = FALSE)
  
  # match cell radius to state-geotype classes by geotype and assumptions  
  RAN.AREA <- RAN.AREA %>%
    # join state-geotype classes to assumptions, by geotype
    left_join(Cell.Parameters, by = "GEOTYPE") %>%
    # join state-geotype classes to radii, by assumptions
    left_join(Cell.Radius, by = c("GEOTYPE", "INDOOR", "CEDR"))
  
  # calculate site area and number of cells needed to cover that area
  RAN.AREA <- mutate(RAN.AREA,
                     # sites area is area of a circle, minus overlap areas
                     SITE.AREA = circ(RADIUS)*(1-CELL.OVERLAP),
                     # no. of cells needed is total area divided by site area, rounded up
                     CELLS.COVERAGE = ceiling(AREA / SITE.AREA))
  
  
  rm(Cell.Parameters, Cell.Radius)
  
  ### CAPACITY DIMENSIONING  ###
  
  # This section calculates the number of capacity cells for each state-geotype class
    
  ## Determine traffic requirements ##
    
  # Read parameters for no. of users per sq km
  Traffic <- read.xlsx(xlsxFile = INPUTS, sheet = "Traffic", startRow = 1, skipEmptyRows = FALSE) 
  # Select and name relevant columns
  Traffic <- select(Traffic, GEOTYPE, DOWNLINK, UPLINK, DOWN.GROWTH, UP.GROWTH, OVERFLOW)
  
  # merge state-geotype classes with the number of users by geotype
  RAN.AREA <- left_join(RAN.AREA, Traffic, by = "GEOTYPE")
  rm(Traffic)

  # Calculate total uplink and downlink traffic for each state-geotype class
  RAN.AREA <- mutate(RAN.AREA,
                     # Total traffic, downlink 
                     TRAFFIC.DOWN = DOWNLINK * (1 +DOWN.GROWTH)^(t-1) * (1-OVERFLOW),                  
                     # Total traffic, uplink 
                     TRAFFIC.UP = UPLINK * (1 +UP.GROWTH)^(t-1) * (1-OVERFLOW)                       
                     ) 
  
  ## Capacity per site ## 
    
  # Read parameters for number of sectors per site
  SECTORS <- read.xlsx(xlsxFile = INPUTS, sheet = "No.Sectors", startRow = 1, skipEmptyRows = FALSE)
  # Select and rename relevant columns
  SECTORS <- select(SECTORS, REGION, VALUE)
  colnames(SECTORS) <- c("GEOTYPE", "NO.SECTORS")
    
  # Read parameters for spectrum 
  SPECTRUM <- read.xlsx(xlsxFile = INPUTS, sheet = "Spectrum", startRow = 1, skipEmptyRows = FALSE)
  row.names(SPECTRUM) <- as.vector(SPECTRUM[,"NAME"])
  
  # Read parameters for spectral efficiency 
  SPEC.EFF <- read.xlsx(xlsxFile = INPUTS, sheet = "SpectralEfficiency", startRow = 1, skipEmptyRows = FALSE)
  # Get the relevant spectral efficiency value for year t
  SPEC.EFF <- filter(SPEC.EFF, YEAR == t)
  row.names(SPEC.EFF) <- as.vector(SPEC.EFF[,"NAME"])
    
  # Join  data
  RAN.AREA <- left_join(RAN.AREA, SECTORS, by = "GEOTYPE")
  
  # Calculate capacity per site, uplink and downlink
  RAN.AREA <- mutate(RAN.AREA,
                     # spectrum parameters
                        SA.DOWN = SPECTRUM["SpectrumAllocationDownlink", "VALUE"],
                        SA.UP = SPECTRUM["SpectrumAllocationUplink", "VALUE"],
                        SE.DOWN = SPEC.EFF["SpectralEfficiencyDownlink", "VALUE"],
                        SE.UP = SPEC.EFF["SpectralEfficiencyUplink", "VALUE"],
                        UTILISATION = SPECTRUM["Utilisation", "VALUE"],
                     # calculate capacity per site = number of sectors x spectral allocation x spectral efficiency x utilisation
                        CAPACITY.DOWN = NO.SECTORS * SA.DOWN * SE.DOWN * UTILISATION,
                        CAPACITY.UP = NO.SECTORS* SA.UP * SE.UP * UTILISATION
                    )
                  
  rm(SPECTRUM, SECTORS)
  
  # Calculate the number of capacity cells
  RAN.AREA <- mutate(RAN.AREA,
                     # Downlink cells = total donwlink traffic divided by total donwlink capacity, rounded up
                     CELLS.DOWN = ceiling((AREA*TRAFFIC.DOWN)/CAPACITY.DOWN),
                     # Uplink cells = total uplink traffic divided by total uplink capacity, rounded up
                     CELLS.UP = ceiling((AREA*TRAFFIC.UP)/CAPACITY.UP),
                     # Get the maximum of uplink and downlink cells
                     CELLS.CAPACITY = pmax(CELLS.DOWN, CELLS.UP),
                     # The total number of cells required is the maximum of coverage cells
                     # and capacity cells
                     CELLS.END = pmax(CELLS.COVERAGE, CELLS.CAPACITY)
                     )
  
  # Drop unnecessary columns
  RAN.AREA <- select(RAN.AREA, STATE, GEOTYPE, AREA, CELLS.COVERAGE, CELLS.CAPACITY, CELLS.END)
    
  ### EXOGENOUS SITES ###

  # This section 'overrides' the number of cells required (as calculated above) if
  # the number of cells is specified exogenously
  
  # import 'exogenous' sites
  SITES <- read.xlsx(xlsxFile = INPUTS, sheet = "Sites", startRow = 1, skipEmptyRows = FALSE)
  # select and name relevent columns
  SITES <- select(SITES, STATE, GEOTYPE, VALUE)
  colnames(SITES) <- c("STATE", "GEOTYPE", "CELLS.EXO")

  # join exogenous sites to state-geotype class by state and geotype
  RAN <- left_join(RAN.AREA, SITES, by = c("STATE", "GEOTYPE"))
  
  # The number of cells equals the number of exogenously specified cells
  RAN$CELLS <- RAN$CELLS.EXO
  # Unless the value is missing, in which case it is equal to the number
  # of cells calculated endogenously (as above)
  RAN$CELLS[is.na(RAN$CELLS)] <- RAN$CELLS.END[is.na(RAN$CELLS)]


  ### COVERAGE ###
  
  # This section 'drops' coverage for state-geotype classes if they are not
  # within the coverage area
  
  # Read coverage parameter
  Coverage <- Parameters["Coverage","VALUE"]
  
  # Remove geotypes that are not covered
  # A geotype is covered if its number exceeds the coverage parameter
  RAN <- filter(RAN, GEOTYPE >= Coverage)
  
  rm(Coverage)
  
  ### OUTPUT ###
  
  # This section defines the output for this function
  # The output is the final number of cells for each state-geotype class
  
  OUTPUT <- RAN['CELLS']

}


# Apply RAN Dimensioning function -----------------------------------------

# This section creates a list/vector of all years (t) within the time horizon
# and applies the RAN dimensioning function for each year

# Define the time horizon
T <- Parameters["TimeHorizon", "VALUE"] 

# Create a list/vector of all years within the time horizon 
T.LIST<- as.list(1:T)
T.VECTOR <- as.data.frame(c(1:T))

# Apply the RAN dimensioning function for each year 
RAN <- lapply(T.LIST,RAN.DIMENSIONING)
# Reshape as data frame and assign column names
RAN <- as.data.frame(RAN)
colnames(RAN) <-c(1:T)

# Rejoin numeric outputs with state-geotype descriptors
RAN <-  cbind(RAN.AREA[c('STATE','GEOTYPE')],RAN)

rm(RAN.DIMENSIONING)

# Change in sites year-to-year --------------------------------------------

# This section calculates the number of additional sites required in each
# state-geotype class in each year.

# Function for calculating year-on-year change
Change <- function(x){
  PROFILE <- RAN[as.character(c(1:T))]
  # For each year t, the number of additional sites is given by the number 
  # of sites in year t, less the number of sites in year t-1
  OUTPUT <- if(x>1){                
    PROFILE[x] - PROFILE[x-1]       
  # Except in year 1, when the number of additional sites is given by 
  # the number of sites in year 1
  }else{                            
    PROFILE[1]                      
  }
}

# Apply the function for calculating year-on-year change for each year 
# in the time horizon
RAN.ADDITIONAL <- lapply(T.LIST,Change)
# Reshape as a data frame and assign column names
RAN.ADDITIONAL <- as.data.frame(RAN.ADDITIONAL)
colnames(RAN.ADDITIONAL) <- c(1:T)

# Rejoin numeric outputs with state-geotype descriptors
RAN.ADDITIONAL <- cbind(RAN[c('STATE','GEOTYPE')], 
                        RAN.ADDITIONAL)

rm(Change)

