#### MNO CAPACITY AUGMENTATION (RAN SITES) #################################

# This file calculates the number of additional MNO sites for each year in the 
# time horizon and the year-on-year change in the number of sites.
# 
# For each state-geotype class and each year, it calculates: capacity 
# per site and overflow traffic. For each geotype, Overflow traffic is estimated 
# as a fixed proportion of total traffic. 
# 
# The number of overflow sites is estimated by dividing overflow traffic by
# capacity per site.


# Read area of each state-geotype class -----------------------------------

RAN.AREA <- read.csv("OUTPUT/SUMMARY.GEOTYPE.csv")


# Define RAN Dimensioning function ----------------------------------------

# This function calculates the number of sites required for each state-geotype 
# class for each year (t) in the time horizon.

MNO.AUGMENTATION <- function(t){
  
  # This section calculates the number of capacity cells for each state-geotype class
  
  ## Determine traffic requirements ##
  
  # Read parameters for traffic per sq km
  Traffic <- read.xlsx(xlsxFile = INPUTS, sheet = "Traffic", startRow = 1, skipEmptyRows = FALSE) 
  # Select and name relevant columns
  Traffic <- select(Traffic, GEOTYPE, DOWNLINK, UPLINK, DOWN.GROWTH, UP.GROWTH, OVERFLOW)
  
  # merge state-geotype classes with traffic by geotype
  RAN.AREA <- left_join(RAN.AREA, Traffic, by = "GEOTYPE")
  rm(Traffic)
  
  # Calculate total uplink and downlink overflow traffic for each state-geotype class
  RAN.AREA <- mutate(RAN.AREA,
                     # Overflow traffic, downlink 
                     TRAFFIC.DOWN = DOWNLINK * (1 +DOWN.GROWTH)^(t-1) * (OVERFLOW),                  
                     # Overflow traffic, uplink 
                     TRAFFIC.UP = UPLINK * (1 +UP.GROWTH)^(t-1) * (OVERFLOW)                       
  ) 
  
  ## Capacity per site ## 
  
  # Read parameters for number of sectors per site
  SECTORS <- read.xlsx(xlsxFile = INPUTS, sheet = "No.Sectors", startRow = 1, skipEmptyRows = FALSE)
  # Select and rename relevant columns
  SECTORS <- select(SECTORS, REGION, VALUE)
  colnames(SECTORS) <- c("GEOTYPE", "NO.SECTORS")
  
  # Read parameters for spectrum 
  SPECTRUM <- read.xlsx(xlsxFile = INPUTS, sheet = "Spectrum", startRow = 1, skipEmptyRows = FALSE)
  row.names(SPECTRUM) <- as.vector(SPECTRUM[,"NAME"])
  
  # Read parameters for spectral efficiency 
  SPEC.EFF <- read.xlsx(xlsxFile = INPUTS, sheet = "SpectralEfficiency", startRow = 1, skipEmptyRows = FALSE)
  # Get the relevant spectral efficiency value for year t
  SPEC.EFF <- filter(SPEC.EFF, YEAR == t)
  row.names(SPEC.EFF) <- as.vector(SPEC.EFF[,"NAME"])
  
  # Join  data
  RAN.AREA <- left_join(RAN.AREA, SECTORS, by = "GEOTYPE")
  
  # Calculate capacity per site, uplink and downlink
  RAN.AREA <- mutate(RAN.AREA,
                     # spectrum parameters
                     SA.DOWN = SPECTRUM["MNOSpectrumDownlink", "VALUE"] ,
                     SA.UP = SPECTRUM["MNOSpectrumUplink", "VALUE"],
                     SE.DOWN = SPEC.EFF["SpectralEfficiencyDownlink", "VALUE"],
                     SE.UP = SPEC.EFF["SpectralEfficiencyUplink", "VALUE"],
                     UTILISATION = SPECTRUM["Utilisation", "VALUE"],
                     # calculate capacity per site = number of sectors x spectral allocation x spectral efficiency x utilisation
                     CAPACITY.DOWN = NO.SECTORS * SA.DOWN * SE.DOWN * UTILISATION,
                     CAPACITY.UP = NO.SECTORS* SA.UP * SE.UP * UTILISATION
  )
  
  rm(SPECTRUM, SECTORS)
  
  # Calculate the number of capacity cells
  RAN.AREA <- mutate(RAN.AREA,
                     # Downlink cells = total donwlink traffic divided by total donwlink capacity, rounded up
                     CELLS.DOWN = ceiling((AREA*TRAFFIC.DOWN)/CAPACITY.DOWN),
                     # Uplink cells = total uplink traffic divided by total uplink capacity, rounded up
                     CELLS.UP = ceiling((AREA*TRAFFIC.UP)/CAPACITY.UP),
                     # Get the maximum of uplink and downlink cells
                     CELLS.OVERFLOW = pmax(CELLS.DOWN, CELLS.UP)
                     # The total number of cells required is the maximum of coverage cells
                     # and capacity cells
  )
  
  # Drop unnecessary columns
  RAN.AREA <- select(RAN.AREA, STATE, GEOTYPE, AREA, CELLS.OVERFLOW)
  
  ### COVERAGE ###
  
  # This section 'drops' coverage for state-geotype classes if they are not
  # within the coverage area
  
  # Read coverage parameter
  Coverage <- Parameters["Coverage","VALUE"]
  
  # Remove geotypes that are not covered
  # A geotype is covered if its number exceeds the coverage parameter
  RAN <- filter(RAN.AREA, GEOTYPE >= Coverage)
  
  rm(Coverage)
  
  ### OUTPUT ###
  
  # This section defines the output for this function
  # The output is the final number of cells for each state-geotype class
  
  OUTPUT <- RAN['CELLS.OVERFLOW']
  
}

# Apply MNO Augmentation function --------------------------------------------

# This section creates a list/vector of all years (t) within the time horizon
# and applies the RAN dimensioning function for each year


# Apply the RAN dimensioning function for each year 
MNO.SITES <- lapply(T.LIST, MNO.AUGMENTATION)
# Reshape as data frame and assign column names
MNO.SITES <- as.data.frame(MNO.SITES)
colnames(MNO.SITES) <-c(1:T)

rm(MNO.AUGMENTATION)

# CALCULATE YEAR-TO-YEAR CHANGE IN NO. SITES ------------------------------

# This section calculates the number of additional MNO sites in each
# state-geotype class in each year.

# Function for calculating year-on-year change
Change <- function(x){
  # For each year t, the number of additional sites is given by the number 
  # of sites in year t, less the number of sites in year t-1
  OUTPUT <- if(x>1){                
    MNO.SITES[x] - MNO.SITES[x-1]  
    # Except in year 1, when the number of additional sites is given by 
    # the number of sites in year 1     
  }else{                            
    MNO.SITES[x]                      
  }
}

# Apply the function for calculating year-on-year change for each year 
# in the time horizon
MNO.SITES.ADDITIONAL <- apply(T.VECTOR,1,Change)
# Reshape as a data frame and assign column names
MNO.SITES.ADDITIONAL <- as.data.frame(MNO.SITES.ADDITIONAL)
colnames(MNO.SITES.ADDITIONAL) <- c(1:T)

# RESHAPE -----------------------------------------------------------------

# This section rejoins numeric outputs from functions with state-geotype 
# descriptors

# Rejoin numeric outputs with state-geotype descriptors
MNO.SITES.ADDITIONAL <- cbind(RAN.AREA[c('STATE','GEOTYPE')], 
                              MNO.SITES.ADDITIONAL)

# Rejoin numeric outputs with state-geotype descriptors
MNO.SITES <- cbind(RAN.AREA[c('STATE','GEOTYPE')], 
                   MNO.SITES)

rm(Change)
