#### ROLLOUT AND REPLACEMENT SCHEDULE ##########################################

# This file creates a rollout and replacement schedule for each capital 
# expenditure item in each state-geotype class. The schedule specifies how many 
# units of that item will be deployed in each year within the time horizon.

# Number of units required ------------------------------------------------

# This section imports inputs relating to the number of units required for 
# each cost item. These fall into three categories:
#   1. The number of units required depends on the number of RAN sites
#   2. The number of units required depends on the number of users
#   3. The number of units required is specified exogenously
# For each item, a profile is generated that describes how many additional
# units are required in each year. 

## SITE ITEMS ##

# This section of the code applies to cost items where the number of units
# required depends on the number of RAN sites

# Read parameters relating to number of units required
SITE.ITEMS <- read.xlsx(xlsxFile = INPUTS, sheet = 'Site.Items' , startRow = 1)
# Select relevant columns
SITE.ITEMS <- select(SITE.ITEMS, NAME, GEOTYPE, VALUE)
# Join with parameters with additional RAN sites each year
SITE.ITEMS <- full_join(RAN.ADDITIONAL, SITE.ITEMS, by = "GEOTYPE")
# Calculate additional number of units of each item required each year
SITE.ITEMS <- cbind(SITE.ITEMS[c("NAME", "STATE","GEOTYPE")], 
                    SITE.ITEMS[as.character(c(1:T))] * SITE.ITEMS$VALUE)

## USER ITEMS ##

# This section of the code applies to cost items where the number of units
# required depends on the number of users

# Read parameters relating to number of units required
# Select relevant columns
USER.ITEMS <- read.xlsx(xlsxFile = INPUTS, sheet = 'User.Items' , startRow = 1)
USER.ITEMS <- select(USER.ITEMS, NAME, GEOTYPE, VALUE)
# Join with number of additional users each year
USER.ITEMS <- full_join(USER.ADDITIONAL, USER.ITEMS, by = "GEOTYPE")
# Calculate additional number of units of each item required each year
USER.ITEMS <- cbind(USER.ITEMS[c("NAME", "STATE","GEOTYPE")], 
                    USER.ITEMS[as.character(c(1:T))] * USER.ITEMS$VALUE)

## CORE ITEMS ##

# This section of the code applies to cost items where the number of units
# required is specified exogenously

# Read parameters relating to number of units required
CORE.ITEMS <- read.xlsx(xlsxFile = INPUTS, sheet = 'Core.Items' , startRow = 1)
# Select relevant columns
CORE.ITEMS <- select(CORE.ITEMS, NAME, STATE, GEOTYPE, VALUE)
# Create profile of when 'additional' units are required. All units are
# required in year 1, with zero units required in subsequent years, so 
# the profile has '1' in first column and '0' in all other years.
CORE.PROFILE <- data.frame(matrix(0, nrow = nrow(CORE.ITEMS), ncol = T))
colnames(CORE.PROFILE) <- c(1:T)
CORE.PROFILE$"1" <- 1
# Calculate total number of units of each item required
CORE.ITEMS <- cbind(CORE.ITEMS[c("NAME", "STATE","GEOTYPE")], 
                    CORE.PROFILE * CORE.ITEMS$VALUE)
rm(CORE.PROFILE)

## MNO AUGMENTATION ITEMS ##

# This section of the code applies to cost items where the number of units
# required depends on the number of additional MNO sites

# Read parameters relating to number of units required
MNO.ITEMS <- read.xlsx(xlsxFile = INPUTS, sheet = 'Aug.Items' , startRow = 1)
# Select relevant columns
MNO.ITEMS <- select(MNO.ITEMS, NAME, GEOTYPE, VALUE)
# Join with parameters with additional RAN sites each year
MNO.ITEMS <- full_join(MNO.SITES.ADDITIONAL, MNO.ITEMS, by = "GEOTYPE")
# Calculate additional number of units of each item required each year
MNO.ITEMS <- cbind(MNO.ITEMS[c("NAME", "STATE","GEOTYPE")], 
                   MNO.ITEMS[as.character(c(1:T))] * MNO.ITEMS$VALUE)


## COMBINE ALL ITEMS INTO ONE DF ##

# This section binds together all item names and the number of units required.
# Note that the number of MNO sites was calculated previously in '04 - Capacity
# Augmentation'.

ITEMS <- bind_rows (SITE.ITEMS, 
                    USER.ITEMS,
                    CORE.ITEMS, 
                    MNO.ITEMS)

rm (SITE.ITEMS, 
    USER.ITEMS,
    MNO.ITEMS,
    CORE.ITEMS)

# Import RRS data ---------------------------------------------------------

# This section reads inputs relating to the rollout and replacement schedule 
# for each item, and merges these inputs with the dataframe containing 
# information on the number of items required over time.

# Read inputs relating to rollout and replacement schedule
RRS <- read.xlsx(xlsxFile = INPUTS, sheet = "Rollout.Replacement", startRow = 1) 
# Select and rename relevant columns
RRS <- select(RRS, NAME, ROLLOUT, REPLACEMENT)
colnames(RRS) <- c("NAME", "ROLLOUT.YEARS", "REP.FREQ")

# Merge dataframe of all items with rollout and replacement schedule for
# each item.
ITEMS <- left_join(ITEMS, RRS, by = "NAME")

# Create RRS schedule -----------------------------------------------------

# This section defines a function that creates a rollout and replacement 
# schedule over the evaluation period for a particular expenditure item. The
# function is applied over all items in each state-geotype class. 

# Define the rollout and replacement function
Rollout <- function(x){
  
  # get the rollout period and replacement frequency for the relevant item (x)
  RO <- as.numeric(ITEMS[x,"ROLLOUT.YEARS"]) # rollout period
  REP<- as.numeric(ITEMS[x,"REP.FREQ"])      # replacement frequency
  
  # This function identified how many of units of item x are required in year t
  # and creates a replacement schedule for those units
  Replacement.profile <- function(t){
    
    # The number of item x required in year t
    NO.ITEMS <- as.numeric(ITEMS[x,as.character(t)])
    
    # Create a vector that describes when those units are replaced - e.g. if 30 
    # units are required in year 1 and must be replaced after three years, that
    # vector would be (30,0,0,30,0,0,30,0,0,...). 
    REPLACEMENT.SCHED <- c(NO.ITEMS, rep(0, times = (REP-1)) )
    REPLACEMENT.SCHED <- c(rep(0, times = t-1), rep(REPLACEMENT.SCHED, times = T))
    
    # Truncate the length of the vector so it is equal to the length of 
    # the evaluation period
    length(REPLACEMENT.SCHED) <- T
    
    # output of this function is the truncated vector
    OUTPUT <- REPLACEMENT.SCHED
    
  }
  
  # Apply the replacement profile function for every year in the time horizon
  REP.PROF <- apply(T.VECTOR, 1, Replacement.profile)
  # Combine all outputs to get a single replacement schedule for item x.
  REP.PROF <- rowSums(as.data.frame(REP.PROF))
  

  # This function takes the replacement schedule for item x, and creates
  # a rollout schedule for the units required in year t
  Rollout.each.year <- function(t){
    # Get the number of units required in year t
    NO.ITEMS <- as.numeric(REP.PROF[t])
    
    # Create a vector that describes when those units are rolled out - e.g. if 15 
    # units are required in year 1 and are rolled out over three years, that
    # vector would be (5,5,5,0,0,0,0,0,0,0,...)
    ROLLOUT.SCHED <- c(rep(0,times = t-1), rep(NO.ITEMS/RO, times = RO), rep(0,times=T))
    
    # Truncate the length of the vector so it is equal to the length of 
    # the evaluation period
    length(ROLLOUT.SCHED) <- T
    
    # Output of this function is the trucated vector
    OUTPUT <- ROLLOUT.SCHED
  }
  
  # Apply the rollout.each.year function for every year in the time horizon
  ROLLOUT.x <- apply(T.VECTOR, 1,Rollout.each.year)
  # Combine all outputs to get a single rollout schedule for item x.
  ROLLOUT.x <- rowSums(as.data.frame(ROLLOUT.x))
 
  # Output of this function is the rollout schedule for item x (which also 
  # accounts for all replacements of item x)
}

# Apply the rollout function for each item for each state-geotype class
N <- as.data.frame(c(1:nrow(ITEMS)))
ROLLOUT <- apply(N,1,Rollout)
# Reshape as dataframe
ROLLOUT <- (as.data.frame(t(ROLLOUT)))
# Assign column names
colnames(ROLLOUT) <- c(1:T)

# Reshape  ----------------------------------------------------------------

# Rejoin numeric outputs with state, geotype and item descriptors
ROLLOUT <- cbind(ITEMS[c('NAME', 'STATE','GEOTYPE')], 
                            ROLLOUT)
# Keep a dataframe with numeric outputs only
NO.ITEMS <- ROLLOUT[as.character(c(1:T))]
