#### CAPITAL EXPENDITURE COSTS ##########################################

# This file calculates total capital expenditures made over the length of
# the evaluation period. Adjustment to capital costs is made for discount
# factors and real cost trends. The residual value of assets at the end
# of the evaluation period is applied as a negative capital expenditure at
# the end of the evaluation period

# CAPEX: Discount Factors -------------------------------------------------

# This section creates a dataframe of discount factors to be applied to 
# capital expenditure costs over time

# Get interest rate
r <- Parameters["DiscRate", "VALUE"]/100 

# Create a vector of discount factors
DISCOUNT.FACTORS <- 1/((1+r)^(T.VECTOR)) 

# Transform  vector into  matrix, then dataframe
DISCOUNT.FACTORS <- matrix(DISCOUNT.FACTORS, nrow = T , ncol = nrow(NO.ITEMS)) 
DISCOUNT.FACTORS <-  as.data.frame(t(DISCOUNT.FACTORS))

# CAPEX:  Cost trends -----------------------------------------------------

# This section creates a dataframe of trended unit costs for all capital 
# expenditures within the evaluation period

# Get per unit cost and trend parameters
COST <- read.xlsx(xlsxFile = INPUTS, sheet = "Costs", startRow = 1, skipEmptyRows = FALSE)

# This function generates a vector of cost trend multipliers over time for a
# given capital expenditure item, n.
Cost.trends <- function(n){
  trend <- COST[n,"TREND"]
  trend <- (1+trend)^(T.VECTOR)
}

# Apply cost trend function to each capital expenditure item
N <- as.data.frame(1:nrow(COST))
COST.TRENDS <- apply(N,1, Cost.trends)
# Reshape as data frame
COST.TRENDS <- as.data.frame(t(COST.TRENDS))
# Assign column names
colnames(COST.TRENDS) <- c(1:T)

rm(Cost.trends, N)

# Get trended costs by multiplying cost by the vector of multipliers
# and rejoin with cost descriptors
COST <- cbind(COST['NAME'], COST$VALUE * COST.TRENDS)

# CAPEX: Match costs to items ---------------------------------------------

# This section matches trended per-unit costs with CAPEX items in each 
# state-geotype class. These costs are then discounted using the dataframe of 
# discount  factors generated above. Total costs are obtained by multiplying 
# discounted unit costs by the number of items in each year.

# Join trended costs with cost items by state-geotype class
CAPEX <- left_join(ROLLOUT[1:3], COST, by = "NAME") 
# Get values only
COST <- CAPEX[as.character(c(1:T))] 

# Obtain total costs, discounted
CAPEX.VALUES <- COST * NO.ITEMS * DISCOUNT.FACTORS

# Residual values: EAV ----------------------------------------------------

# This section calculates the equivalent annual value (EAV) of each 
# capital expenditure item over time.

# Define m, an arbitrarily large number greater than T. This is the 
# 'time horizon' for calculating residual values, and should be 
# sufficiently long to capture the lifespan of all assets rolled out 
# within the evaulation period.
m <- 100 
# M is the set of  all years within the time horizon
M <- as.data.frame(c(1:m))

# Join state-geotype data with rollout and replacement schedule
ROLLOUT <- left_join(ROLLOUT, RRS, by = "NAME")

# This function creats a vector of EAVs for all units of item x over time
EAV <- function(x){
  
  # Get the rollout period and replacement frequency
  RO <- as.numeric(ROLLOUT[x,"ROLLOUT.YEARS"])
  REP <- as.numeric(ROLLOUT[x,"REP.FREQ"])
  
  # This function creates a vector of EAVs for all units of item x
  # rolled out in year t
  EAV.profile <- function(t){
    # Get the number of units rolled out in year t
    NO.ITEMS <- as.numeric(ROLLOUT[x,as.character(t)])
    # Create a profile of the EAV for each year in M
    EAV.PROFILE <- rep(1/REP, times = REP) * NO.ITEMS
    EAV.PROFILE <- c(rep(0, times = t-1), EAV.PROFILE, rep(0, times = m))
    # Truncate the vector to length m
    length(EAV.PROFILE) <- m
    # Output is a vector that describes the profile of EAV over time
    OUTPUT <- EAV.PROFILE
  }
  
  # Apply EAV.profile function to each year within the time horizon 
  # (This captures only those rollouts/replacements that begin within
  # the time horizon)
  EAV.PROFILE <- apply(T.VECTOR, 1, EAV.profile)
  # Combine all outputs to get a single EAV profile for item x.  
  EAV.PROFILE <- rowSums(as.data.frame(EAV.PROFILE))
  
}

# apply EAV function to all items in each state-geotype class
N <- as.data.frame(c(1:nrow(ROLLOUT)))
EAV.PROFILE <- apply (N, 1, EAV)
# Reshape as dataframe
EAV.PROFILE <- as.data.frame(t(EAV.PROFILE))

# Residual values: Discount factors ---------------------------------------

# This section applies discount factors to the EAVs over time

# Get interest rate
r <- Parameters["DiscRate", "VALUE"]/100 

# Create a vector of discount factors
DISCOUNT.FACTORS <- 1/((1+r)^(M)) 

# Get a matrix / dataframe of discount factors
DISCOUNT.FACTORS <- matrix(DISCOUNT.FACTORS, nrow = m , ncol = nrow(ROLLOUT)) 
DISCOUNT.FACTORS <-  as.data.frame(t(DISCOUNT.FACTORS))

# Residual values: Cost trends --------------------------------------------

# This section creates a dataframe of trended unit costs for all EAVs.

# Get per unit cost and trends parameters
COST <- read.xlsx(xlsxFile = INPUTS, sheet = "Costs", startRow = 1, skipEmptyRows = FALSE)

# This function generates a vector of cost trend multipliers over time for a
# given capital expenditure item, n.
Cost.trends <- function(n){
  trend <- COST[n,"TREND"]
  trend <- (1+trend)^(M)
}

# Apply cost trend function to each capital expenditure item
N <- as.data.frame(1:nrow(COST))
COST.TRENDS <- apply(N,1, Cost.trends)
# Reshape as data frame
COST.TRENDS <- as.data.frame(t(COST.TRENDS))
# Assign column names
colnames(COST.TRENDS) <- c(1:m)

rm(Cost.trends, M)

# Get trended costs by multiplying cost by the vector of multipliers
# and rejoin with cost descriptors
COST.TRENDS <- cbind(COST['NAME'], COST$VALUE * COST.TRENDS)

# Merge trended costs with each cost item in each state-geotype class
COST.TRENDS <- left_join(ROLLOUT[1:3], COST.TRENDS, by = "NAME") 

# Get values only (remove descriptive columns)
COST.TRENDS <- COST.TRENDS[as.character(c(1:m))] 

# Residual values: calculate residual value -------------------------------

# This section calculates the residual value of each item in each 
# state-geotype class. Cost trends and discount factors are applied
# to the EAVs. The residual value is obtained as the sum of all EAVs
# outside the time horizon.


# Apply cost trends and discount factors to EAV
EAV.VALUES <- COST.TRENDS * DISCOUNT.FACTORS * EAV.PROFILE

# For each item, get the sum of all EAVs outside the time horizon
EAV.VALUES <- rowSums(EAV.VALUES[(T+1):m])

# The residual value is applied as a negative CAPEX at the end of the 
# time horizon
CAPEX.VALUES$RESID.EAV <- -(EAV.VALUES)

# Reshape -----------------------------------------------------------------

# This section computes total CAPEX for each item in each state-geotype
# class and then re-organises the CAPEX dataframe so that outputs are 
# easily read. 

# Total CAPEX is the sum of CAPEX in each time period
CAPEX.VALUES$TOTAL <- rowSums(CAPEX.VALUES)

# Join with state names and geotype names
CAPEX <-  ROLLOUT[c("NAME", "STATE", "GEOTYPE")] %>%
  left_join(., STATE.CODES, by = "STATE") %>%
  left_join(., GEOTYPE.CODES, by = "GEOTYPE") %>%
  cbind(.,CAPEX.VALUES)

# Order rows by state, then geotype
attach(CAPEX)
CAPEX <- CAPEX[order(STATE,-GEOTYPE),]
detach(CAPEX)
