### OPEX ###################################################################

OPEX <- read.xlsx(xlsxFile = INPUTS, sheet = "OPEX", startRow = 1)
OPEX <- select(OPEX, NAME, PER.CENT, FIXED)

# Number of units ---------------------------------------------------------

# This section imports inputs relating to the number of units required for 
# each cost item. These fall into three categories:
#   1. The number of units required depends on the number of RAN sites
#   2. The number of units required depends on the number of users
#   3. The number of units required is specified exogenously
# For each item, a profile is generated that describes the total number of
# units required in each year. 

## SITE ITEMS ##

# This section of the code applies to cost items where the number of units
# required depends on the number of RAN sites

# Read parameters relating to number of units required
SITE.ITEMS <- read.xlsx(xlsxFile = INPUTS, sheet = 'Site.Items' , startRow = 1)
# Select relevant columns
SITE.ITEMS <- select(SITE.ITEMS, NAME, GEOTYPE, VALUE)
# Join with parameters RAN site numbers
SITE.ITEMS <- full_join(RAN, SITE.ITEMS, by = "GEOTYPE")
# Calculate total number of units of each item required
SITE.ITEMS <- cbind(SITE.ITEMS[c("NAME", "STATE","GEOTYPE")], 
                    SITE.ITEMS[as.character(c(1:T))] * SITE.ITEMS$VALUE)

## USER ITEMS ##

# This section of the code applies to cost items where the number of units
# required depends on the number of users

# Read parameters relating to number of units required
USER.ITEMS <- read.xlsx(xlsxFile = INPUTS, sheet = 'User.Items' , startRow = 1)
# Select relevant columns
USER.ITEMS <- select(USER.ITEMS, NAME, GEOTYPE, VALUE)
# Join with No. of users
USER.ITEMS <- full_join(USER, USER.ITEMS, by = "GEOTYPE")
# Calculate total number of units of each item required
USER.ITEMS <- cbind(USER.ITEMS[c("NAME", "STATE","GEOTYPE")], 
                    USER.ITEMS[as.character(c(1:T))] * USER.ITEMS$VALUE)

## CORE ITEMS ##

# This section of the code applies to cost items where the number of units
# required is specified exogenously

# Read parameters relating to number of units required
CORE.ITEMS <- read.xlsx(xlsxFile = INPUTS, sheet = 'Core.Items' , startRow = 1)
# Select relevant columns
CORE.ITEMS <- select(CORE.ITEMS, NAME, STATE, GEOTYPE, VALUE)
# Create profile of what proportion of units are required in each year
# (i.e. all units are required in every year)
CORE.PROFILE <- data.frame(matrix(1, nrow = nrow(CORE.ITEMS), ncol = T))
colnames(CORE.PROFILE) <- c(1:T)
# Calculate total number of units of each item required
CORE.ITEMS <- cbind(CORE.ITEMS[c("NAME", "STATE","GEOTYPE")], 
                    CORE.PROFILE * CORE.ITEMS$VALUE)

## MNO AUGMENTATION ITEMS ##

# This section of the code applies to cost items where the number of units
# required depends on the number of additional MNO sites

# Read parameters relating to number of units required
MNO.ITEMS <- read.xlsx(xlsxFile = INPUTS, sheet = 'Aug.Items' , startRow = 1)
# Select relevant columns
MNO.ITEMS <- select(MNO.ITEMS, NAME, GEOTYPE, VALUE)
# Join with parameters with additional RAN sites each year
MNO.ITEMS <- full_join(MNO.SITES, MNO.ITEMS, by = "GEOTYPE")
# Calculate additional number of units of each item required each year
MNO.ITEMS <- cbind(MNO.ITEMS[c("NAME", "STATE","GEOTYPE")], 
                   MNO.ITEMS[as.character(c(1:T))] * MNO.ITEMS$VALUE)

## COMBINE ALL ITEMS INTO ONE DF ##

# This section binds together all item names and the number of units required.
# Note that the number of MNO sites was calculated previously in '04 - Capacity
# Augmentation'.

ITEMS <- bind_rows (SITE.ITEMS, 
                    USER.ITEMS,
                    MNO.ITEMS,
                    CORE.ITEMS)

OPEX <- left_join(ITEMS, OPEX, by = "NAME")

rm (SITE.ITEMS, 
    USER.ITEMS,
    MNO.ITEMS,
    CORE.ITEMS, 
    CORE.PROFILE,
    ITEMS)

# Accounting for initial rollout ------------------------------------------

# Because operating costs are tied to capital expenditure items, OPEX
# 'phases in' in line with the intial rollout of CAPEX items. This section
# generates, for each CAPEX item, a profile that describes what how many 
# units are operational over time. 

# Join number of units with rollout and replacement schedules
OPEX <- left_join(OPEX, RRS, by = "NAME")


# This function generates, for item x, a profile that describes what 
# proportion of the total is operational over time. 
CumulativeRollout <- function(x){
  
  # Get rollout time for item x
  RO <- as.numeric(OPEX[x,"ROLLOUT.YEARS"])
 
  # Generate a profile that describes what proportion has been rolled out
  # For example, if an item were to be rolled out uniformly over four years, 
  # its profile would be (0.25,0.5,0.75,1,1,1,1,1,...)
  ROLLOUT.CUM <- c(seq(from = 1/RO, to = 1, by = 1/RO), rep(1, times = T))
  
  # Truncate the length of the vector so it is equal to the length of 
  # the evaluation period
  length(ROLLOUT.CUM) <-T
 
  # Output of this function is the truncated vector
  OUTPUT <- ROLLOUT.CUM
  
}

# Apply the Cumulative rollout function to every cost item
N <- as.data.frame(c(1:nrow(COST.TRENDS)))
CUMULATIVE.ROLLOUT <- apply(N, 1, CumulativeRollout)
# reshape as data frame
CUMULATIVE.ROLLOUT <- as.data.frame(t(CUMULATIVE.ROLLOUT))

# Multiply the cumulative rollout profile by the total number of units
# to be rolled out. This gives a profile in terms of no. of units. 
CUMULATIVE.ROLLOUT <- OPEX[as.character(c(1:T))]  * CUMULATIVE.ROLLOUT

# Calculate OPEX (undiscounted) --------------------------------------------

# This section calculates (undiscounted) OPEX costs for each item in each 
# state-geotype class over the time horizon. OPEX is expressed in per unit
# terms, in one of two ways:
#   1. It is expressed as a per cent of the inital unit cost
#   2. It is expressed as a fixed value per unit

# Merge number of units with unit cost by name
OPEX <- left_join(OPEX, COST, by = "NAME")

# OPEX PER CENT - For each item in each year, OPEX PER CENT is calculated as:
# Some Percentange x Initial Unit Cost x Number of units 
OPEX.PC <- OPEX$PER.CENT * OPEX$VALUE * CUMULATIVE.ROLLOUT 

# OPEX FIXED - For each item in each year, OPEX FIXED is calculated as:
# Some Fixed Value x Number of Units
OPEX.FIX <- OPEX$FIXED * CUMULATIVE.ROLLOUT  

# Total OPEX for each item is the sum of OPEX PER CENT and OPEX FIXED.
OPEX.VALUES <- OPEX.PC + OPEX.FIX

rm(OPEX.PC, OPEX.FIX)

# Discounting -------------------------------------------------------------

# This section creates a dataframe of discount factors to be applied to 
# capital expenditure costs over time

# Create a vector of discount factors
DISCOUNT.FACTORS <- 1/((1+r)^(T.VECTOR)) 

# Multiply OPEX values by discount factors
OPEX.VALUES <- t(as.data.frame(t(OPEX.VALUES)) * DISCOUNT.FACTORS)

# Reshape as dataframe and assign column names
OPEX.VALUES <- as.data.frame(OPEX.VALUES)
colnames(OPEX.VALUES) <- c(1:T)

rm(DISCOUNT.FACTORS)

# Reshape -----------------------------------------------------------------

# This section computes total OPEX for each item in each state-geotype
# class and then re-organises the CAPEX dataframe so that outputs are 
# easily read. 

# Total OPEX is the sum of OPEX in each time period
OPEX.VALUES$TOTAL <-rowSums(OPEX.VALUES)

# Join with state names and geotype names
OPEX <- OPEX[c('NAME', 'STATE','GEOTYPE')] %>%
        left_join(., STATE.CODES, by = "STATE") %>%
        left_join(., GEOTYPE.CODES, by = "GEOTYPE") %>%
        cbind(., OPEX.VALUES)

# Order rows by state, then geotype
attach(OPEX)
OPEX <- OPEX[order(STATE,-GEOTYPE),]
detach(OPEX)


rm(OPEX.VALUES)